<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\AddressesResource;
use App\Models\Address;
use Illuminate\Http\Request;
use JWTAuth;

class AddressesController extends Controller
{
    public function index()
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $rows = Address::whereUserId($user->id)->get();
        return AddressesResource::collection($rows)->additional(['value' => true]);

    }

    public function store(Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'address' => 'required',
            'title' => 'required',
            'lat' => 'required',
            'lng' => 'required'
        ]);
        if ($validator->passes()) {
            $request['user_id'] = $user->id;
            Address::create($request->all());
            $rows = Address::whereUserId($user->id)->get();
            return AddressesResource::collection($rows)->additional(['value' => true]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function destroy($id)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        Address::destroy($id);
        $rows = Address::whereUserId($user->id)->get();
        return AddressesResource::collection($rows)->additional(['value' => true]);

    }

    public function update($id, Request $request)
    {
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $address = Address::find($id);
        $address['lat'] = $request['lat'];
        $address['lng'] = $request['lng'];
        $address['address'] = $request['address'];
        $address['title'] = $request['title'];
        $address->update();
        $rows = Address::whereUserId($user->id)->get();
        return AddressesResource::collection($rows)->additional(['value' => true]);

    }
}
