<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Resources\OffersResource;
use App\Http\Resources\OrderRescource;
use App\Http\Resources\WorkersResource;
use App\Models\Notification;
use App\Models\Offer;
use App\Models\Order;
use App\Models\OrderCategory;
use App\Models\OrderImage;
use App\Models\Setting;
use App\Models\UserToken;
use App\User;
use Edujugon\PushNotification\PushNotification;
use Illuminate\Http\Request;
use JWTAuth;

class OrdersController extends Controller
{
    public function order(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'categories' => 'required',
//            'images' => 'required',
            'title' => 'required',
            'description' => 'required',
//            'city_id' => 'required|exists:cities,id',
//            'lat' => 'required',
//            'lng' => 'required',
//            'address' => 'required',
            'date' => 'required',
//            'start_time' => 'required',
//            'finish_time' => 'required',
            'address_id' => 'required'
        ]);


        if ($validator->passes()) {
            $order = new Order();
            $order->user_id = $user->id;
            $order->title = $request['title'];
            $order->description = $request['description'];
            $order->address_id = $request['address_id'];
            $order->date = $request['date'];
            $order->start_time = $request['start_time'];
            $order->finish_time = $request['finish_time'];
            $order->status = 'new';
            $order->payment_type = 'cash';
            $order->code = uniqid();
            $order->save();
            if ($request['categories']) {
                foreach (explode(',', $request['categories']) as $category) {
                    OrderCategory::create([
                        'order_id' => $order->id,
                        'category_id' => $category
                    ]);
                }
            }
            $i = 1;
            if ($request['images']) {
                foreach ($request['images'] as $single_image) {
                    $filename = $i . time() . '.' . $single_image->getClientOriginalExtension();
                    $single_image->move(public_path('images/orders'), $filename);
                    OrderImage::create([
                        'order_id' => $order->id,
                        'image' => $filename
                    ]);
                    $i++;

                }
            }


            // notification for workers


            $categories_array = explode(',', $request['categories']);
            $matched_workers = User::where('type', 'worker')->where('status', 'active')->get()->filter(function ($d_worker) use ($categories_array) {
                $worker_categories = $d_worker->worker_categories->pluck('id');
                $result = array_intersect($categories_array, $worker_categories->toArray());
                return count($result) == count($categories_array);
            });

            $workers = $matched_workers->pluck('id');


            foreach ($workers as $worker) {
                $notification = new Notification();
                $notification['user_id'] = $worker;
                $notification['body->ar'] = " لديكم طلب جديد  ";
                $notification['body->en'] = "You have a new Order ";
                $notification['type'] = 'new_order';
                $notification['order_id'] = $order->id;
                $notification->save();
            }

            $tokens = UserToken::whereIn('user_id', $workers)->get();
            $push = new PushNotification('fcm');
            foreach ($tokens as $token) {

                $title = " لديكم طلب جديد  ";
                $body = " لديكم طلب جديد  ";

                if ($token->device_type == 'ios') {
                    $msg = [
                        'data' => [
                            'title_ar' => " لديكم طلب جديد  ",
                            'title_en' => "You have a new Order ",
                            'body_ar' => " لديكم طلب جديد  ",
                            'body_en' => "You have a new Order ",
                            'type' => 'new_order',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'notification' => [
                            'title_ar' => " لديكم طلب جديد  ",
                            'title_en' => "You have a new Order ",
                            'body_ar' => " لديكم طلب جديد  ",
                            'body_en' => "You have a new Order ",
                            'type' => 'new_order',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                } else {
                    $msg = [
                        'data' => [
                            'title_ar' => " لديكم طلب جديد  ",
                            'title_en' => "You have a new Order ",
                            'body_ar' => " لديكم طلب جديد  ",
                            'body_en' => "You have a new Order ",
                            'type' => 'new_order',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                }

            }


            $data = new OrderRescource($order);
            return response()->json(['value' => true, 'data' => $data]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function search_workers(Request $request)
    {

        $validator = \Validator::make($request->all(), [
            'categories' => 'required',
            'lat' => 'required',
            'lng' => 'required'
        ], [
            'lat.required' => \Lang::get('data.openGPS'),
            'lng.required' => \Lang::get('data.openGPS'),
        ]);
        if ($validator->passes()) {
            $categories_array = explode(',', $request['categories']);


            $lat = $request->lat;
            $lng = $request->lng;


            $matched_workers = User::select(\DB::raw('*, ( 6367 * acos( cos( radians(' . $lat . ') ) * cos( radians( lat ) ) * cos( radians( lng ) - radians(' . $lng . ') ) + sin( radians(' . $lat . ') ) * sin( radians( lat ) ) ) ) AS distance'))
                ->having('distance', '<', 100)
                ->orderBy('distance')->where('type', 'worker')->where('status', 'active')->get()->filter(function ($d_worker) use ($categories_array) {
                    $worker_categories = $d_worker->worker_categories->pluck('id');
                    $result = array_intersect($categories_array, $worker_categories->toArray());
                    return count($result) > 0;
                });


            return WorkersResource::collection($matched_workers)->additional(['value' => true]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }


    }

    public function offer_response(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'offer_id' => 'required|exists:offers,id',
            'status' => 'required|in:approve,decline'
        ]);
        if ($validator->passes()) {
            $offer = Offer::find($request['offer_id']);
            $offer['status'] = $request['status'];
            $offer->update();
            if ($request['status'] == 'approve') {
                $setting = Setting::first();
                Order::where('id', $offer->order_id)->update([
                    'offer_id' => $offer->id,
                    'price_offer' => $offer->price,
                    'total_price' => (integer)$offer->price + (integer)($offer->price * $setting->fee / 100),
                    'status' => 'in_progress',
                    'worker_id' => $offer->worker_id
                ]);


                $order = Order::find($offer->order_id);
                // send notification to worker
                $notification = new Notification();
                $notification['user_id'] = $offer->worker_id;
                $notification['order_id'] = $offer->order_id;
                $notification['body->ar'] = " تمت الموافقة علي عرضكم علي طلب رقم  $order->code";
                $notification['body->en'] = " Your offer for order no. $order->code is approved ";
                $notification['type'] = 'client_accept';
                $notification->save();
                $tokens = UserToken::where('user_id', $offer->worker_id)->get();
                $push = new PushNotification('fcm');
                foreach ($tokens as $token) {


                    $title = $notification->body['ar'];
                    $body = $notification->body['ar'];

                    if ($token->device_type == 'ios') {
                        $msg = [
                            'data' => [
                                'title_ar' => $notification['body->ar'],
                                'title_en' => $notification['body->en'],
                                'body_ar' => $notification['body->ar'],
                                'body_en' => $notification['body->en'],
                                'type' => 'client_accept',
                                'sound' => 'default',
                                'title' => $title,
                                'body' => $body,
                                'order_id' => $order->id,
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'notification' => [
                                'title_ar' => $notification['body->ar'],
                                'title_en' => $notification['body->en'],
                                'body_ar' => $notification['body->ar'],
                                'body_en' => $notification['body->en'],
                                'type' => 'client_accept',
                                'sound' => 'default',
                                'title' => $title,
                                'body' => $body,
                                'order_id' => $order->id,
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();

                    } else {
                        $msg = [
                            'data' => [
                                'title_ar' => $notification['body->ar'],
                                'title_en' => $notification['body->en'],
                                'body_ar' => $notification['body->ar'],
                                'body_en' => $notification['body->en'],
                                'type' => 'client_accept',
                                'sound' => 'default',
                                'title' => $title,
                                'body' => $body,
                                'order_id' => $order->id,
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();

                    }

                }
                $data = new OrderRescource($order);
                return response()->json(['value' => true, 'data' => $data]);


            } else {
                $order = Order::find($offer->order_id);

                $offer->delete();
                $data = new OrderRescource($order);
                return response()->json(['value' => true, 'data' => $data]);

            }


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);

        }
    }

    public function single_order($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $order = Order::find($id);
        $data = new OrderRescource($order);
        return response()->json(['value' => true, 'data' => $data]);
    }

    public function pay_order(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make(\request()->all(), [
            'order_id' => 'required|exists:orders,id',
            'payment_type' => 'required|in:wallet,visa,cash'
        ]);

        if ($validator->passes()) {
            $order = Order::find($request['order_id']);
            $order_offer_price = $order->price_offer; // offer price
            $order_all_price = $order->total_price; // offer + fee
            $worker = User::find($order->worker_id);

            $order['payment_type'] = 'cash';
            $order['system_money'] = $order_all_price;


//
//            if ($request['payment_type'] == 'cash') {
//
//
//                $order['payment_type'] = 'cash';
//                $order['system_money'] = $order_all_price;
//
//            }
//
//
//

//
//            elseif ($request['payment_type'] == 'visa') {
//
//                $add_commission = $order_offer_price - ($order_offer_price * $commission / 100);
//
//                $worker->wallet += $add_commission;
//                $additional_fee = $order_all_price - $order_offer_price;
//
//                $order['system_money'] = $additional_fee;
//
//                $order['payment_type'] = 'visa';
//
//            }
//            elseif ($request['payment_type'] == 'wallet') {
//
//
//                $order_owner = User::find($order->user_id);
//                $owner_wallet = $order_owner->wallet * $setting->point_money;
//
//                if ($owner_wallet >= $order_all_price) {
//
//                    $price_in_points = $order_all_price / $setting->point_money;
//                    $order_owner->wallet -= $price_in_points;
//                    $order_owner->update();
//
//
//                    $add_commission = $order_offer_price - ($order_offer_price * $commission / 100);
//
//                    $worker->wallet += $add_commission;
//                    $additional_fee = $order_all_price - $order_offer_price;
//
//                    $order['system_money'] = $additional_fee;
//                    $order['payment_type'] = 'wallet';
//                    $order_owner->wallet += $setting->order_point;
//
//
//                } else {
//                    return response()->json(['value' => false, 'msg' => \Lang::get('date.wallet_less')]);
//                }
//
//
//            }

            $order->update();
            $worker->update();

            $data = new OrderRescource($order);
            return response()->json(['value' => true, 'data' => $data]);

        } else {
            return response()->json(['value' => true, 'msg' => $validator->errors()->first()]);
        }


    }

    public function client_orders($flag)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        if ($flag == 'in_progress') {
            $rows = Order::whereIn('status', ['in_progress', 'worker_in_way'])->whereUserId($user->id)->orderByDesc('id')->paginate(10);
        } elseif ($flag == 'finish') {
            $rows = Order::whereUserId($user->id)->whereIn('status', ['finish', 'client_finish', 'worker_finish'])->orderByDesc('id')->paginate(10);
        } else {
            $rows = Order::whereUserId($user->id)->whereStatus($flag)->orderByDesc('id')->paginate(10);
        }
        return OrderRescource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function update_order(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [

            'city_id' => 'exists:cities,id',
            'order_id' => 'required|exists:orders,id'

        ]);
        if ($validator->passes()) {
            $order = Order::find($request['order_id']);
            $order->title = $request['title'] ? $request['title'] : $order->title;
            $order->description = $request['description'] ? $request['description'] : $order['description'];
            $order->city_id = $request['city_id'] ? $request['city_id'] : $order->city_id;
            $order->lat = $request['lat'] ? $request['lat'] : $order->lat;
            $order->lng = $request['lng'] ? $request['lng'] : $order->lng;
            $order->address = $request['address'] ? $request['address'] : $order->address;
            $order->date = $request['date'] ? $request['date'] : $order->date;
            $order->start_time = $request['start_time'] ? $request['start_time'] : $order->start_time;
            $order->finish_time = $request['finish_time'] ? $request['finish_time'] : $order->finish_time;

            $order->update();

            $i = 1;
            if ($request->images) {
                OrderImage::whereOrderId($order->id)->delete();
                foreach ($request['images'] as $single_image) {
                    $filename = $i . time() . '.' . $single_image->getClientOriginalExtension();
                    $single_image->move(public_path('images/orders'), $filename);
                    OrderImage::create([
                        'order_id' => $order->id,
                        'image' => $filename
                    ]);
                    $i++;

                }
            }
            $data = new OrderRescource($order);
            return response()->json(['value' => true, 'data' => $data]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }


    }

    public function cancel_order($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        Order::where('id', $id)->update([
            'status' => 'canceled'
        ]);

        return response()->json(['value' => true, 'data' => \Lang::get('data.order_canceled')]);
    }

    public function delete_order($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        Order::destroy($id);

        return response()->json(['value' => true, 'data' => \Lang::get('data.deleted')]);
    }

    public function worker_home()
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $rows = Order::whereStatus('new')->orderByDesc('id')->paginate(10);
        return OrderRescource::collection($rows)->additional(['value' => true, 'code' => 200]);

    }

    public function add_offer(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id',
            'price' => 'required',
            'comment' => 'required'

        ]);
        if ($validator->passes()) {
            $order = Order::find($request['order_id']);
            Offer::create([
                'worker_id' => $user->id,
                'order_id' => $request['order_id'],
                'price' => $request['price'],
                'comment' => $request['comment'],
                'status' => 'waiting'
            ]);

            // notification for owner


            // send notification to worker
            $notification = new Notification();
            $notification['user_id'] = $order->user_id;
            $notification['order_id'] = $order->id;
            $notification['body->ar'] = " لديكم عرض طلب لطلب رقم $order->code ";
            $notification['body->en'] = " Your order no. $order->code has a new offer ";
            $notification['type'] = 'offer';
            $notification->save();
            $tokens = UserToken::where('user_id', $order->user_id)->get();
            $push = new PushNotification('fcm');
            foreach ($tokens as $token) {

                $title = $notification->body['ar'];
                $body = $notification->body['ar'];

                if ($token->device_type == 'ios') {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification['body->ar'],
                            'title_en' => $notification['body->en'],
                            'body_ar' => $notification['body->ar'],
                            'body_en' => $notification['body->en'],
                            'type' => 'offer',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'notification' => [
                            'title_ar' => $notification['body->ar'],
                            'title_en' => $notification['body->en'],
                            'body_ar' => $notification['body->ar'],
                            'body_en' => $notification['body->en'],
                            'type' => 'offer',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                } else {
                    $msg = [
                        'data' => [
                            'title_ar' => $notification['body->ar'],
                            'title_en' => $notification['body->en'],
                            'body_ar' => $notification['body->ar'],
                            'body_en' => $notification['body->en'],
                            'type' => 'offer',
                            'sound' => 'default',
                            'title' => $title,
                            'body' => $body,
                            'order_id' => $order->id,
                            'click_action' => 'FCM_PLUGIN_ACTIVITY',
                        ],
                        'priority' => 'high',
                    ];
                    $push->setMessage($msg)
                        ->setDevicesToken($token->token)
                        ->send();

                }

            }


            $data = new OrderRescource($order);
            return response()->json(['value' => true, 'data' => $data]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function single_offer($id)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $offer = Offer::find($id);
        $data = new OffersResource($offer);
        return response()->json(['value' => true, 'data' => $data]);

    }

    public function update_offer(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        $validator = \Validator::make($request->all(), [
            'offer_id' => 'required|exists:offers,id',
            'price' => 'required',
            'comment' => 'required'

        ]);

        if ($validator->passes()) {
            Offer::where('id', $request['offer_id'])->update([
                'price' => $request['price'],
                'comment' => $request['comment']
            ]);
            $offer = Offer::find($request['offer_id']);
            $data = new OffersResource($offer);
            return response()->json(['value' => true, 'data' => $data]);


        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }


    }

    public function delete_offer($id)
    {
        Offer::destroy($id);
        return response()->json(['value' => true, 'data' => \Lang::get('data.delete_done')]);
    }

    public function worker_orders($flag)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }

        if ($flag == 'waiting') {
            $worker_offers = Offer::where('worker_id', $user->id)->whereStatus('waiting')->pluck('order_id');
            $rows = Order::whereIn('id', $worker_offers)->whereStatus('new')->orderByDesc('id')->paginate(10);
        } elseif ($flag == 'in_progress') {
            $rows = Order::whereIn('status', ['in_progress', 'worker_in_way'])->whereWorkerId($user->id)->orderByDesc('id')->paginate(10);
        } elseif ($flag == 'finish') {
            $rows = Order::whereIn('status', ['finish', 'client_finish', 'worker_finish'])->whereWorkerId($user->id)->orderByDesc('id')->paginate(10);
        } else {
            $rows = Order::where('status', $flag)->whereWorkerId($user->id)->orderByDesc('id')->paginate(10);

        }

        return OrderRescource::collection($rows)->additional(['value' => true, 'code' => 200]);


    }

    public function change_order_status(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'status' => 'required|in:worker_finish,canceled,worker_in_way,client_finish',
            'order_id' => 'required|exists:orders,id'
        ]);
        if ($validator->passes()) {

            $order = Order::find($request['order_id']);

            if ($request['status'] == 'client_finish') {
                $finish_code = rand(1111, 9999);
                $order->finish_code = $finish_code;


                // send code for worker


                $client_name = $order->user->username;
                // send notification to worker
                $notification = new Notification();
                $notification['user_id'] = $order->worker_id;
                $notification['order_id'] = $order->id;
                $notification['body->ar'] = "قام العميل $client_name بإرسال كود الإنتهاء علي طلب رقم $order->code و كود الإنتهاء هو $finish_code ";
                $notification['body->en'] = "Client $client_name finished order no . $order->code and Finish code is $finish_code";
                $notification['type'] = 'order_finish';
                $notification->save();
                $tokens = UserToken::where('user_id', $order->worker_id)->get();
                $push = new PushNotification('fcm');
                foreach ($tokens as $token) {

                    $title = $notification->body['ar'];
                    $body = $notification->body['ar'];
                    if ($token->device_type == 'ios') {
                        $msg = [
                            'data' => [
                                'title_ar' => $notification['body->ar'],
                                'title_en' => $notification['body->en'],
                                'body_ar' => $notification['body->ar'],
                                'body_en' => $notification['body->en'],
                                'type' => 'order_finish',
                                'sound' => 'default',
                                'title' => $title,
                                'body' => $body,
                                'order_id' => $order->id,
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'notification' => [
                                'title_ar' => $notification['body->ar'],
                                'title_en' => $notification['body->en'],
                                'body_ar' => $notification['body->ar'],
                                'body_en' => $notification['body->en'],
                                'type' => 'order_finish',
                                'sound' => 'default',
                                'title' => $title,
                                'body' => $body,
                                'order_id' => $order->id,
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();

                    } else {
                        $msg = [
                            'data' => [
                                'title_ar' => $notification['body->ar'],
                                'title_en' => $notification['body->en'],
                                'body_ar' => $notification['body->ar'],
                                'body_en' => $notification['body->en'],
                                'type' => 'order_finish',
                                'sound' => 'default',
                                'title' => $notification['body->ar'],
                                'body' => $notification['body->ar'],
                                'order_id' => $order->id,
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();

                    }

                }


            } elseif ($request['status'] == 'worker_finish') {
                if ($order->finish_code == null) {
                    return response()->json(['value' => false, 'msg' => \Lang::get('data.client_finish_first')]);
                } else {

                    $order->status = $request['status'];

                }
            } elseif ($request['status'] == 'worker_in_way') {


                // notification for owner
                $worker_name = $order->worker->username;
                // send notification to worker
                $notification = new Notification();
                $notification['user_id'] = $order->user_id;
                $notification['order_id'] = $order->id;
                $notification['body->ar'] = " الفني $worker_name في طريقة لكم ";
                $notification['body->en'] = "Worker $worker_name is in his way";
                $notification['type'] = 'worker_in_way';
                $notification->save();
                $tokens = UserToken::where('user_id', $order->user_id)->get();
                $push = new PushNotification('fcm');
                foreach ($tokens as $token) {

                    $title = $notification->body['ar'];
                    $body = $notification->body['ar'];
                    if ($token->device_type == 'ios') {
                        $msg = [
                            'data' => [
                                'title_ar' => $notification['body->ar'],
                                'title_en' => $notification['body->en'],
                                'body_ar' => $notification['body->ar'],
                                'body_en' => $notification['body->en'],
                                'type' => 'worker_in_way',
                                'sound' => 'default',
                                'title' => $title,
                                'body' => $body,
                                'order_id' => $order->id,
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'notification' => [
                                'title_ar' => $notification['body->ar'],
                                'title_en' => $notification['body->en'],
                                'body_ar' => $notification['body->ar'],
                                'body_en' => $notification['body->en'],
                                'type' => 'worker_in_way',
                                'sound' => 'default',
                                'title' => $title,
                                'body' => $body,
                                'order_id' => $order->id,
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();

                    } else {
                        $msg = [
                            'data' => [
                                'title_ar' => $notification['body->ar'],
                                'title_en' => $notification['body->en'],
                                'body_ar' => $notification['body->ar'],
                                'body_en' => $notification['body->en'],
                                'type' => 'worker_in_way',
                                'sound' => 'default',
                                'title' => $title,
                                'body' => $body,
                                'order_id' => $order->id,
                                'click_action' => 'FCM_PLUGIN_ACTIVITY',
                            ],
                            'priority' => 'high',
                        ];
                        $push->setMessage($msg)
                            ->setDevicesToken($token->token)
                            ->send();

                    }

                }


                $order->status = $request['status'];
            } else {
                $order->status = $request['status'];
            }

            $order->update();


            $data = new OrderRescource($order);

            return response()->json(['value' => true, 'data' => $data]);

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);
        }
    }

    public function worker_finish_code(Request $request)
    {
        $lang = \request()->header('Accept-Language', 'ar');
        \App::setLocale($lang);
        if (!$user = JWTAuth::parseToken()->authenticate()) {
            return response()->json(['value' => false, 'msg' => \Lang::get('data.wrong_token'), 'code' => 401]);
        }
        $validator = \Validator::make($request->all(), [
            'order_id' => 'required|exists:orders,id',
            'code' => 'required'
        ]);
        if ($validator->passes()) {
            $order = Order::find($request['order_id']);
            if ($order->finish_code == $request['code']) {

                $order->status = 'finish';
                $order->update();
                $data = new OrderRescource($order);
                return response()->json(['value' => true, 'data' => $data, 'message' => \Lang::get('data.code_correct')]);


            } else {
                return response()->json(['value' => true, 'data' => \Lang::get('data.code_error')]);

            }

        } else {
            return response()->json(['value' => false, 'msg' => $validator->errors()->first()]);

        }
    }

}
